@@ -1,14 +1,28 @@
-#![cfg_attr(test, feature(test))]
-
-//!This crate contains parser combinators, roughly based on the Haskell library [parsec](http://hackage.haskell.org/package/parsec).
+//! This crate contains parser combinators, roughly based on the Haskell library [parsec](http://hackage.haskell.org/package/parsec).
 //!
-//!A parser in this library can be described as a function which takes some input and if it
-//!is succesful, returns a value together with the remaining input.
-//!A parser combinator is a function which takes one or more parsers and returns a new parser.
-//!For instance the `many` parser can be used to convert a parser for single digits into one that
-//!parses multiple digits.
+//! A parser in this library can be described as a function which takes some input and if it
+//! is succesful, returns a value together with the remaining input.
+//! A parser combinator is a function which takes one or more parsers and returns a new parser.
+//! For instance the `many` parser can be used to convert a parser for single digits into one that
+//! parses multiple digits.
 //!
-//!# Examples:
+//!# Overview
+//!
+//! This library is currently split into three modules.
+//!
+//! * `primitives` contains the `Parser` trait as well as various structs dealing with input
+//! streams and errors.
+//!
+//! * `combinator` contains the before mentioned parser combinators and thus contains the main
+//! building blocks for creating any sort of more complex parsers. It consists of free functions as
+//! well as a the `ParserExt` trait which provides a few functions which are more naturally used
+//! through method calls.
+//!
+//! * `char` is the last module. It provides parsers specifically working with streams of
+//! characters. As a few examples it has parsers for accepting digits, letters or whitespace.
+//!
+//!
+//!# Examples
 //!
 //!```
 //! extern crate parser_combinators as parser_combinators;
@@ -31,14 +45,14 @@
 //! }
 //!```
 //!
-//!If we need a parser that is mutually recursive we can define a free function which internally
-//!can in turn be used as a parser (Note that we need to explicitly cast the function, this should
-//!not be necessary once changes in rustc to make orphan checking less restrictive gets implemented)
+//! If we need a parser that is mutually recursive we can define a free function which internally 
+//! can in turn be used as a parser (Note that we need to explicitly cast the function, this should
+//! not be necessary once changes in rustc to make orphan checking less restrictive gets implemented)
 //!
-//!`expr` is written fully general here which may not be necessary in a specific implementation
-//!The `Stream` trait is predefined to work with array slices, string slices and iterators
-//!meaning that in this case it could be defined as
-//!`fn expr(input: State<&str>) -> ParseResult<Expr, &str>`
+//! `expr` is written fully general here which may not be necessary in a specific implementation
+//! The `Stream` trait is predefined to work with array slices, string slices and iterators
+//! meaning that in this case it could be defined as
+//! `fn expr(input: State<&str>) -> ParseResult<Expr, &str>`
 //!
 //!```
 //! extern crate parser_combinators as parser_combinators;
@@ -75,9 +89,6 @@
 //! }
 //!```
 
-#[cfg(test)]
-extern crate test;
-
 #[doc(inline)]
 pub use primitives::{Parser, ParseResult, ParseError, from_iter};
 #[doc(inline)]
@@ -102,8 +113,7 @@ pub use char::{
 pub use combinator::{
     between,
     chainl1,
-    choice_slice,
-    choice_vec,
+    choice,
     many,
     many1,
     optional,
@@ -325,28 +335,6 @@ r"
         }
     }
 
-static LONG_EXPR: &'static str =
-r"(3 * 4) + 2 * 4 * test + 4 * aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa
-* (a + 2 * ((476128368 + i * (((3 * 4) + 2 * 4 * test + 4 * aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa
-* (a + 2 * ((476128368 + i * ((476128368 + 21476)+ 21476)) + 21476)) * 2123 * 214 + (476128368 + 21476) * hello + 42 + 21476)+ 21476)) + 21476)) * 2123 * 214 + (476128368 + 21476) * hello + 42 +
-(3 * 4) + 2 * 4 * test + 4 * aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa
-* (a + 2 * ((476128368 + i * (((3 * 4) + 2 * 4 * test + 4 * aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa
-* (a + 2 * ((476128368 + i * ((476128368 + 21476)+ 21476)) + 21476)) * 2123 * 214 + (476128368 + 21476) * hello + 42
-+ 21476)+ 21476)) + 21476)) * 2123 * 214 + (476128368 + 21476) * hello + 42";
-    #[bench]
-    fn bench_expression(bench: &mut ::test::Bencher) {
-
-        let result = parser(term)
-            .parse(LONG_EXPR);
-        assert!(result.is_ok()); 
-        assert_eq!(result.unwrap().1, "");
-        bench.iter(|| {
-            let result = parser(term)
-                .parse(LONG_EXPR);
-            let _ = ::test::black_box(result);
-        })
-    }
-
     #[test]
     fn sep_by_error_consume() {
         let mut p = sep_by::<Vec<_>, _, _>(string("abc"), satisfy(|c| c == ','));
@@ -407,7 +395,7 @@ r"(3 * 4) + 2 * 4 * test + 4 * aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa
             try(string("Fig")),
             try(string("Grape")),
         ];
-        let mut parser = choice_slice(&mut fruits[..]);
+        let mut parser = choice(&mut fruits);
         assert_eq!(parser.parse("Apple"), Ok(("Apple", "")));
         assert_eq!(parser.parse("Banana"), Ok(("Banana", "")));
         assert_eq!(parser.parse("Cherry"), Ok(("Cherry", "")));
